package de.intevation.flys.artifacts.model.fixings;

import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.CallContext;

import de.intevation.flys.artifacts.FLYSArtifact;

import de.intevation.flys.artifacts.access.FixAnalysisAccess;

import de.intevation.flys.artifacts.model.CalculationResult;
import de.intevation.flys.artifacts.model.DataFacet;
import de.intevation.flys.artifacts.model.FacetTypes;
import de.intevation.flys.artifacts.model.Parameters;

import de.intevation.flys.artifacts.states.DefaultState.ComputeType;

import de.intevation.flys.utils.KMIndex;

import org.apache.log4j.Logger;

/**
 * Facet to show average W values for Q sectors.
 *
 * @author <a href="mailto:raimund.renkert@intevation.de">Raimund Renkert</a>
 */
public class FixLongitudinalDeviationFacet
extends      DataFacet
implements   FacetTypes {

    /** House logger. */
    private static Logger logger = Logger.getLogger(FixLongitudinalDeviationFacet.class);

    /** Trivial Constructor. */
    public FixLongitudinalDeviationFacet() {
    }


    public FixLongitudinalDeviationFacet(
        int ndx,
        String name,
        String description)
    {
        super(
            ndx,
            name,
            description,
            ComputeType.ADVANCE,
            null,
            null);
    }


    /**
     * Returns the data this facet requires.
     *
     * @param artifact the owner artifact.
     * @param context  the CallContext.
     *
     * @return the data as KMIndex.
     */
    @Override
    public Object getData(Artifact artifact, CallContext context) {
        logger.debug("FixLongitudinalDeviationFacet.getData");

        if (artifact instanceof FLYSArtifact) {
            FLYSArtifact flys = (FLYSArtifact)artifact;
            FixAnalysisAccess access = new FixAnalysisAccess(flys);

            CalculationResult res =
                (CalculationResult) flys.compute(context,
                                                 ComputeType.ADVANCE,
                                                 false);

            FixAnalysisResult result = (FixAnalysisResult) res.getData();

            KMIndex<AnalysisPeriod []> kmPeriods = result.getAnalysisPeriods();

            if (kmPeriods == null) {
                logger.warn("No analysis periods found.");
                return null;
            }

            Parameters params = result.getParameters();

            KMIndex<double[]> kmIndex = new KMIndex<double[]>();
            for (KMIndex.Entry<AnalysisPeriod[]> entry: kmPeriods) {
                double km = entry.getKm();
                double[] stdDev =
                    params.interpolate("km", km, new String[] {"std-dev"});

                if(stdDev == null) {
                    logger.warn("getData: stdDev == null");
                    continue;
                }
                kmIndex.add(km, stdDev);
            }

            return kmIndex;
        }
        else {
            logger.warn("Artifact is no instance of FLYSArtifact.");
            return null;
        }
    }


    /**
     * Create a deep copy of this Facet.
     * @return a deep copy.
     */
    @Override
    public FixLongitudinalDeviationFacet deepCopy() {
        FixLongitudinalDeviationFacet copy = new FixLongitudinalDeviationFacet();
        copy.set(this);
        return copy;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
